##
# Beginning implementation of a tree data structure with multiple
# and arbitrary number of roots or N roots. Structure can be used
# to model business logic, sequence of events, or ingredients.
##

  ##
  # Danny Pham
  # 12/10/12
  ##

class MultirootTree
  include Enumerable
  attr_reader :content, :children, :parents

  def initialize(content)
    @content = content
    @children = []
    @parents = []
  end

  #add a child to the children array, instance variable
  def add_child(child)
    if child.instance_of? MultirootTree
      @children << child unless @children.include? child
      child.add_parent(self) unless child.parents.include?(self)
    end
  end

  #adds a list of MultirootTrees with each successive 
  #MultirootTree being the child of the previous
  def add_descendant_line(*descendants)
    max = descendants.size
    num = 0
    first = self
    second = descendants[num]
    
    while (num < descendants.size)
      first.add_child(second)
      first = second
      second = descendants[num + 1]
      num += 1
    end
  end

  #severs tree connections going down descendant line
  def disowns(child)
    if child.is_a? MultirootTree
      @children -= [child]
      child.emancipates_from(self) if child.parents.include?(self)
    end
  end

  #severs tree connections going up the ancestry
  def emancipates_from(parent)
    @parents -= [parent]
    parent.disowns(self) if parent.children.include?(self) 
  end

  #adding parents is particularly necessary because any one 
  #node can have N parents, not just one
  def add_parent(parent)
    @parents << parent unless @parents.include? parent
    parent.add_child(self) unless parent.children.include?(self)
  end

  #adds immediate parents, not an ancestor line
  def add_parents(*parents)
    parents.each do |parent| 
      self.add_parent(parent)
      parent.add_child(self) unless parent.children.include?(self)
    end
  end

  #returns roots/greatest parents
  def roots
    roots ||= []
    if @parents.flatten.size == 0
      roots << self
    else
      @parents.each {|parent| roots << parent.roots}
    end
    roots.flatten
  end

  #returns "youngest" children
  def floor
    floor = []
    if @children.flatten.size == 0
      floor << MultirootTree.new(self.content)
    else
      @children.each {|child| floor << child.floor}
    end
    floor.flatten
  end

  #returns value of node
  def inspect
    @content.inspect
  end
end